
import io
import json

from . import Document
from beanie.odm.documents import PydanticObjectId
from pydantic import model_validator

from .meta import TimestampMixin

from AgentService.utils.convert import string_to_uuid


class Storage(TimestampMixin, Document):
    vector_store_id: str
    key: str

    @classmethod
    async def get_packed(cls, key: str) -> io.BytesIO:
        storage = await Storage.find_one(Storage.key == key)
        storage_items = await StorageItem.find_many(StorageItem.storage_id == storage.id).to_list()

        data = [
            {
                "text": item.data,
                "metadata": {"id": item.key_id}
            }
            for item in storage_items
        ]

        buffer = io.BytesIO()
        buffer.write((json.dumps(data, ensure_ascii=False)).encode("utf-8"))
        buffer.seek(0)

        return buffer


class StorageItem(TimestampMixin, Document):
    storage_id: PydanticObjectId
    data: dict

    key_id: str

    @model_validator(mode="before")
    def set_unique_id(self, values):
        if "key_id" in values or not values["key_id"]:
            return values

        data_key = values["data"].get("key")
        if data_key:
            values["key_id"] = (values["storage_id"], data_key)

        else:
            values["key_id"] = (values["storage_id"], str(values["data"]))

        values["key_id"] = string_to_uuid(''.join(values["key_id"]))
        return values






from datetime import datetime, timezone


def group_by(target, k=5) -> list:
    return [target[i:i+k] for i in range(0, len(target), k)]


def now() -> int:
    return round(datetime.now(timezone.utc).timestamp())




import os.path
from loguru import logger
import sys


def setup_logger(logs_path, log_level):
    log_path = os.path.join(logs_path, "debug.log")

    fmt = "<g>{time}</> | <lvl>{level}</> | <c>{extra[classname]}:{function}:{line}</> - {message}"

    logger.remove()
    logger.configure(extra={"classname": "None"})
    logger.add(log_path, backtrace=True, diagnose=True, level="DEBUG", format=fmt, rotation="1 day")
    logger.add(sys.stdout, backtrace=True, diagnose=True, level=log_level, format=fmt)
    logger.info("Started logging successfully")




import click
from loguru import logger as log

from AgentService.agent import Agent, AgentTool
from AgentService.config import Config
from AgentService.app import start_app


@click.group()
def group():
    pass


@group.command('start', help="Command that starts AgentService project")
def start_project():
    log.info(f"Starting AgentService project")

    agents = Agent.__subclasses__()
    if not len(agents):
        log.critical(f"No agent can be found")
        return

    first_agent = agents[0]
    log.info(f"Found {len(agents)} agents. Using first one: {first_agent.__name__}")

    tools = list(map(lambda x: x(), AgentTool.__subclasses__()))
    log.info(f"Found {len(tools)} tools: {tools}")

    config = Config()

    agent = first_agent(
        openai_key=config.openai_key,
        tools=tools
    )
    config.set_agent(agent)

    start_app()




import enum


class MessageType(enum.Enum):
    system = "system"
    user = "user"
    assistant = "assistant"
    assistant_skip = "assistant_skip"
    tools = "tools"
    tool_answer = "tool_answer"




from typing import Dict, List, Optional
from pydantic import BaseModel

from AgentService.types.storage import StorageItem
from AgentService.enums.response import ResposneStatus


class GetStorageRequest(BaseModel):
    key: str


class GetStorageResponse(BaseModel):
    data: List[StorageItem]
    description: Optional[str] = None
    status: ResposneStatus


class AddStorageRequest(BaseModel):
    key: str
    data: List[Dict] | Dict


class AddStorageResponse(BaseModel):
    description: Optional[str] = None
    status: ResposneStatus


class RemoveStorageRequest(BaseModel):
    key: str
    data: List[Dict] | Dict


class RemoveStorageResponse(BaseModel):
    description: Optional[str] = None
    status: ResposneStatus


class UpdateStorageRequest(BaseModel):
    key: str
    data: Dict
    new_data: Dict


class UpdateStorageResponse(BaseModel):
    description: Optional[str] = None
    status: ResposneStatus




import AgentService
from AgentService.utils.templates import process_template


import click
import os
from shutil import copy


@click.group()
def group():
    pass


@group.command('add_tool', help="Command to add the tool to AgentService project")
@click.argument('tool', type=str)
@click.option('-t', '--template', default="default")
def add_middleware(tool: str, template: str):
    current_path = os.getcwd()
    templates_dir = os.path.join(AgentService.__path__[0], "templates", "tool")

    tool_path = os.path.join(current_path, "tools", tool.lower())
    if os.path.isdir(tool_path):
        return print(f"Error: tool {tool} has already been added in {tool_path}")

    tool_template = os.path.join(templates_dir, f"{template}.py.tmpl")
    if not os.path.isfile(tool_template):
        return print(f"Error: no such template {template}")

    new_tool_path = os.path.join(current_path, "tools", f"{tool.lower()}.py.tmpl")
    copy(tool_template, new_tool_path)

    process_template(new_tool_path, tool_name=tool.capitalize())

    init_file = os.path.join(current_path, "tools", "__init__.py")
    with open(init_file, "a") as f:
        f.write(f"from .{tool.lower()} import {tool.capitalize()}Tool\n")

    return print(f"Added {tool.capitalize()}Tool in {new_tool_path}")




import enum
from typing import Any
import json


class BaseJsonifed:
    fields = []

    def to_dict(self):
        pass

    def to_json(self):
        return json.dumps(self.to_dict())


class JsonifiedProperty(BaseJsonifed):
    field = "base"

    def to_dict(self):
        try:
            return self.__getattribute__(self.field)

        except AttributeError:
            raise KeyError(f"Cant find field with name: {self.field} in object: {self}")


class Jsonified(BaseJsonifed):
    fields: list[str | JsonifiedProperty] = []

    def __process_dict(self, obj):
        res = {}

        for key in obj:
            if isinstance(obj[key], list):
                res.update({
                    key: self.__process_list(obj[key])
                })

            elif isinstance(obj[key], dict):
                res.update({
                    key: self.__process_dict(obj[key])
                })

            elif issubclass(obj[key].__class__, BaseJsonifed):
                res.update({
                    key: obj[key].to_dict()
                })

            elif issubclass(obj[key].__class__, enum.Enum):
                res.update({
                    key: obj[key].value
                })

            else:
                res.update({
                    key: obj[key]
                })

        return res

    def __process_list(self, obj):
        res = []

        for i in obj:
            if isinstance(i, list):
                res.append(self.__process_list(i))

            elif isinstance(i, dict):
                res.append(self.__process_dict(i))

            elif issubclass(i.__class__, BaseJsonifed):
                res.append(i.to_dict())

            elif issubclass(i.__class__, enum.Enum):
                res.append(i.value)

            else:
                res.append(i)

        return res

    def to_dict(self) -> dict:
        export = {}

        for field in self.fields:

            if isinstance(field, str):
                try:
                    obj = self.__getattribute__(field)

                except AttributeError:
                    raise KeyError(f"Cant find field with name: {field} in object: {self}")

            else:
                obj = field

            if isinstance(obj, list):
                data = {
                    field: self.__process_list(obj)
                }

            elif isinstance(obj, dict):
                data = {
                    field: self.__process_dict(obj)
                }

            elif issubclass(obj.__class__, BaseJsonifed):
                data = {
                    field: obj.to_dict()
                }

            elif issubclass(obj.__class__, enum.Enum):
                data = {
                    field: obj.value
                }

            else:
                data = {
                    field: obj
                }

            export.update(data)

        return export

    def update(self, key: str, value: Any) -> None:
        try:
            obj = self.__getattribute__(key)

        except AttributeError:
            return

        if isinstance(obj, list):
            obj.append(value)

        else:
            obj.update({key: value})

    def to_str(self):
        return json.dumps(
            self.to_dict(),
            sort_keys=True,
            indent=4
        )




from typing import Optional, Dict, List
from pydantic import BaseModel, Field

from AgentService.types import AgentResponse, ChatToolState, ToolAnswer
from AgentService.enums.response import ResposneStatus


class SendMessageRequest(BaseModel):
    chat_id: str
    text: Optional[str] = None
    context: Dict = Field(default_factory=dict)
    tool_answers: List[ToolAnswer] = Field(default_factory=list)


class SendMessageResponse(BaseModel):
    data: Optional[AgentResponse] = None
    description: Optional[str] = None
    status: ResposneStatus


class GetStatesRequest(BaseModel):
    chat_id: str


class GetStatesResponse(BaseModel):
    data: list[ChatToolState] = Field(default_factory=list)
    description: Optional[str] = None
    status: ResposneStatus




import configparser
import json
import os
import importlib

from AgentService.utils.singleton import SingletonMeta
from AgentService.agent import Agent


class Config(metaclass=SingletonMeta):
    def __init__(self, project_path):
        config_path = os.path.join(project_path, "agent.cfg")
        config = configparser.ConfigParser()
        config.read(config_path)

        self.agent = None
        self.agent_data = None
        self.tools_data = None
        self.secrets_data = None
        self.is_nocode = False

        if os.path.isfile("agent.json"):
            self.agent_data = json.load(open("agent.json", "r"))
            self.tools_data = self.agent_data.pop("tools")
            self.secrets_data = self.agent_data.pop("secrets")
            self.is_nocode = True

        self.project_name = config["project"]["name"]
        self.agent_path = config["project"]["agent_source"]
        self.tools_path = config["project"]["tools_source"]
        self.app_host = config["app"]["host"]
        self.app_port = int(config["app"]["port"])
        self.log_level = config["logging"]["level"]
        self.log_path = config["logging"]["path"]

        if self.is_nocode:
            self.openai_key = self.secrets_data.get("openai_key")
            self.db_name = self.secrets_data.get("db_name", "AgentService")
            self.db_user = self.secrets_data.get("db_uri")

        else:
            importlib.import_module(self.agent_path)
            importlib.import_module(self.tools_path)

            self.openai_key = os.getenv("openai_key")
            self.db_name = os.getenv("db_name", default="AgentService")
            self.db_uri = os.getenv("db_uri")

    def set_agent(self, agent: Agent):
        self.agent = agent




import copy
import click
from loguru import logger as log

from AgentService.agent import Agent, AgentTool
from AgentService.config import Config
from AgentService.app import start_app


@click.group()
def group():
    pass


@group.command('start', help="Command that starts AgentService project")
def start_project():
    log.info(f"Starting AgentService project")

    cls = type("Agent0", (Agent, ), Config().agent_data)
    agents = [cls]

    first_agent = agents[0]
    log.info(f"Found {len(agents)} agents. Using first one: {first_agent.__name__}")

    tools = []
    for i, tool_raw in enumerate(Config().tools_data):
        cls = type(f"{AgentTool.__name__}{i}", (AgentTool, ), tool_raw)
        tools.append(cls())

    log.info(f"Found {len(tools)} tools: {tools}")

    agent = first_agent(
        openai_key=Config().openai_key,
        tools=tools
    )
    Config().set_agent(agent)

    start_app()




from AgentService.types import Tool
from typing import Any


class BaseEvent:
    data: Any

    def __str__(self):
        fields = ', '.join([
            f"{key}={self.__dict__[key]}"
            for key in self.__dict__ if not key.startswith("__")
        ])

        return f"{self.__class__.__name__}({fields})"


class ToolEvent(BaseEvent):
    def __init__(self, tool_name: str, tool: Tool, chat_id: str):
        self.tool_name = tool_name
        self.tool = tool
        self.chat_id = chat_id




import json
from loguru import logger
from beanie.operators import And, In

from openai import AsyncOpenAI
from openai.types.responses import ResponseFunctionToolCall, ResponseOutputMessage, ResponseOutputText

from AgentService.types import (
    Chat, ChatToolState,
    AgentResponse, Message,
    Storage, StorageItem,
    Tool, ToolAnswer
)

from AgentService.enums.chat import ChatStatus, ChatToolStateStatus
from AgentService.enums.agent import AgentResponseType
from AgentService.enums.message import MessageType

from AgentService.agent.tool import AgentTool


class Agent:
    model: str = "gpt-4.1-nano"
    temperature: float = 1.0
    max_tokens: int = 2048
    top_p: float = 1.0

    instructions: str = "You are a helpful assistant"

    system_prompts: list[str] = []
    prompt: str = "{text}"

    is_one_shot: bool = False

    def __init__(
        self,
        openai_key: str,
        tools: list[AgentTool] = None
    ):

        self.log = logger.bind(classname=self.__class__.__name__)

        self.client = AsyncOpenAI(api_key=openai_key)

        self.tools = tools if tools else []
        self.tools_schema = [tool.openai_model_dump for tool in self.tools]

        self.system_prompts = list(map(lambda x: x.replace("\t", ""), self.system_prompts))
        self.instructions = self.instructions.replace("\t", "")

        self.log.debug(f"Created gpt like tools -> {self.tools_schema}")

    async def __system_prompt(self, i: int, context: dict) -> str:
        return self.system_prompts[i].format(**context)

    async def __prompt(self, text: str, context: dict) -> str:
        return self.prompt.format(text=text, **context)

    async def __generate(self, chat: Chat) -> AgentResponse:
        chat.status = ChatStatus.generating
        await chat.save()

        messages = await Message.find_many(Message.chat_id == chat.id).to_list()

        gpt_messages = []
        for message in messages:
            data = message.openai_model_dump

            if isinstance(data, list):
                for row in data:
                    gpt_messages.append(row)

            else:
                gpt_messages.append(data)

        tools_schema = self.tools_schema[:]

        storage_item = await StorageItem.find_one()
        if storage_item:
            storages = await Storage.find_many().to_list()

            tools_schema.append(
                {
                    "type": "file_search",
                    "vector_store_ids": list(map(lambda x: x.id, storages))
                }
            )

        try:
            response = await self.client.responses.create(
                model=self.model,
                input=gpt_messages,
                instructions=self.instructions,
                temperature=self.temperature,
                max_output_tokens=self.max_tokens,
                top_p=self.top_p,
                tools=tools_schema,
                tool_choice="auto"
            )

        except Exception as err:
            self.log.debug(json.dumps(gpt_messages, indent=2, ensure_ascii=False))
            self.log.debug(json.dumps(tools_schema, indent=2, ensure_ascii=False))

            raise err

        responses = []
        for i in response.output:
            if i.__class__ not in [ResponseFunctionToolCall, ResponseOutputMessage]:
                self.log.warning(f"Unsupported response type -> {i.__class__}")
                continue

            responses.append(i)

        if len(responses) != 1:
            self.log.debug(json.dumps(response.model_dump(), indent=2, ensure_ascii=False))
            raise TypeError(f"Got many response types in 1 output -> {responses}")

        if isinstance(responses[0], ResponseFunctionToolCall):
            tools = [
                Tool(
                    tool_id=i.call_id,
                    name=i.name,
                    arguments=i.arguments
                )
                for i in responses
            ]

            states = [
                ChatToolState(
                    chat_id=chat.id,
                    status=ChatToolStateStatus.in_progress,
                    tool_id=tool.tool_id,
                    name=tool.name,
                    arguments=tool.arguments
                )
                for tool in tools
            ]

            await ChatToolState.insert_many(states)

            return AgentResponse(
                type=AgentResponseType.tools,
                content=tools
            )

        elif isinstance(responses[0], ResponseOutputMessage):
            texts = []
            for i in responses:
                for j in i.content:
                    if j.__class__ not in [ResponseOutputText]:
                        self.log.warning(f"Unsupported content type -> {j.__class__}")
                        continue

                    texts.append(j.text)

            return AgentResponse(
                type=AgentResponseType.answer,
                content="\n\n".join(texts)
            )

        else:
            self.log.debug(json.dumps(response.model_dump(), indent=2, ensure_ascii=False))
            raise TypeError(f"Unsupported response type -> {responses[0]}")

    async def generate(self, chat: Chat) -> AgentResponse:
        try:
            return await self.__generate(chat)

        except Exception as err:
            self.log.exception(err)

            return AgentResponse(
                type=AgentResponseType.answer,
                content=str(err)
            )

    async def get_chat(self, chat_id: str, context: dict = {}) -> Chat:
        chat = await Chat.find_one(
            And(
                Chat.chat_id == chat_id,
                In(Chat.status, [ChatStatus.created, ChatStatus.idle, ChatStatus.tools])
            )
        )

        if not chat:
            chat = Chat(
                chat_id=chat_id,
                status=ChatStatus.created,
                data=context
            )
            await chat.save()

        system_message = await Message.find_one(
            And(
                Message.chat_id == chat.id,
                Message.type == MessageType.system
            )
        )
        if not system_message:
            messages = []

            for i in range(len(self.system_prompts)):
                system_message = Message(
                    chat_id=chat.id,
                    text=await self.__system_prompt(i, context),
                    type=MessageType.system
                )
                messages.append(system_message)

            await Message.insert_many(messages)

        return chat

    async def proccess_answer(self, answer: AgentResponse, chat: Chat) -> AgentResponse:
        if answer.type == AgentResponseType.tools:
            bot_message = Message(
                chat_id=chat.id,
                type=MessageType.tools,
                tools=answer.content
            )
            chat.status = ChatStatus.tools

        elif answer.type == AgentResponseType.answer:
            bot_message = Message(
                chat_id=chat.id,
                type=MessageType.assistant,
                text=answer.content
            )
            chat.status = ChatStatus.finished if self.is_one_shot else ChatStatus.idle

        else:
            raise ValueError(f"wrong answer type got {answer.type}, expected {AgentResponseType} like")

        await bot_message.insert()
        await chat.save()

        return answer

    async def skip_tools(self, chat: Chat) -> str:
        messages = await Message.find_many(Message.chat_id == chat.id).to_list()
        last_message: Message = messages[-1]

        if not last_message.tools:
            chat.status = ChatStatus.idle
            await chat.save()

            return chat

        new_messages = []
        for tool in last_message.tools:
            tool_state = await ChatToolState.find_one(
                And(
                    ChatToolState.chat_id == chat.id,
                    ChatToolState.tool_id == tool.tool_id,
                )
            )
            if tool_state:
                tool_state.status = ChatToolStateStatus.completed
                await tool_state.save()

            new_messages.append(
                Message(
                    chat_id=chat.id,
                    type=MessageType.tool_answer,
                    tool_answer=ToolAnswer(
                        tool_id=tool.tool_id,
                        name=tool.name,
                        text="Tool call skipped."
                    )
                )
            )

        new_messages.append(
            Message(
                chat_id=chat.id,
                type=MessageType.assistant,
                text="skip"
            )
        )
        await Message.insert_many(new_messages)

        return chat

    async def answer_text(self, chat_id: str, text: str, context: dict = {}) -> AgentResponse:
        chat = await self.get_chat(chat_id, context)
        if chat.status == ChatStatus.tools:
            chat = await self.skip_tools(chat)

        user_message = Message(
            chat_id=chat.id,
            text=await self.__prompt(text, context),
            type=MessageType.user
        )
        await user_message.insert()

        answer = await self.generate(chat=chat)
        await self.proccess_answer(answer, chat)

        return answer

    async def answer_tools(self, chat_id: str, tool_answers: list[ToolAnswer] = None) -> AgentResponse:
        chat = await self.get_chat(chat_id)

        if chat.status != ChatStatus.tools:
            return AgentResponse(type=AgentResponseType.answer, content="No tools to answer")

        new_messages = []
        for tool_answer in tool_answers:
            tool_state = await ChatToolState.find_one(
                And(
                    ChatToolState.chat_id == chat.id,
                    ChatToolState.tool_id == tool_answer.tool_id,
                )
            )
            if tool_state:
                tool_state.status = ChatToolStateStatus.completed
                await tool_state.save()

            new_messages.append(Message(
                chat_id=chat.id,
                type=MessageType.tool_answer,
                tool_answer=tool_answer
            ))

        await Message.insert_many(new_messages)

        answer = await self.generate(chat=chat)
        await self.proccess_answer(answer, chat)

        return answer

    async def answer(self, chat_id: str, text: str = None, context: dict = {}, tool_answers: list[ToolAnswer] = None) -> AgentResponse:
        self.log.debug(f"Answer: {chat_id = }, {text = }, {context = }, {tool_answers = }")

        if text:
            return await self.answer_text(chat_id, text, context)

        elif len(tool_answers):
            return await self.answer_tools(chat_id, tool_answers)

        else:
            raise ValueError("Need text or tool answers to answer")

    async def create_storage(self, key: str) -> Storage:
        from AgentService.config import Config

        vector_store = await self.client.vector_stores.create(name="@".join([Config().project_name, key]))
        self.log.info(f"Created storage -> {vector_store.id}")

        return Storage(
            vector_store_id=vector_store.id,
            key=key
        )

    async def update_storage(self, data: str, storage_id: str):
        files = await self.client.vector_stores.files.list(vector_store_id=storage_id)
        for file in files.data:
            await self.client.vector_stores.files.delete(
                vector_store_id=storage_id,
                file_id=file.id
            )
            self.log.info(f"Removed file from storage -> {storage_id}@{file.id}")

        await self.client.vector_stores.file_batches.upload_and_poll(
            vector_store_id=storage_id,
            files=[("data.json", data)]
        )
        self.log.info(f"Added data to storage -> {storage_id}")




from string import Template
from os import rename


def process_template(path, **kwargs):
    with open(path, "rb") as f:
        raw = f.read().decode("utf8")

    content = Template(raw).substitute(**kwargs)

    new_path = path[:-len('.tmpl')] if path.endswith(".tmpl") else path

    if path.endswith(".tmpl"):
        rename(path, new_path)

    with open(new_path, "wb") as f:
        f.write(content.encode("utf8"))




import fastapi
from beanie.operators import And

from AgentService.config import Config

from AgentService.types.chat import ChatToolState, Chat
from AgentService.enums.chat import ChatToolStateStatus

from .models import (
    SendMessageRequest, SendMessageResponse,
    GetStatesRequest, GetStatesResponse
)


chat_router = fastapi.APIRouter(prefix="/chat")


@chat_router.post("")
async def send_message(request: SendMessageRequest) -> SendMessageResponse:
    agent = Config().agent

    response = await agent.answer(
        chat_id=request.chat_id,
        text=request.text,
        context=request.context,
        tool_answers=list(map(lambda x: x.dict(), request.tool_answers))
    )

    return SendMessageResponse(
        data=response,
        status="ok"
    )


@chat_router.get("/states")
async def get_states() -> GetStatesResponse:
    states = await ChatToolState.find_all().to_list()

    return GetStatesResponse(
        data=states,
        status="ok"
    )


@chat_router.get("/state")
async def get_state(request: GetStatesRequest) -> GetStatesResponse:
    chat = await Chat.find_one(Chat.chat_id == request.chat_id)

    if not chat:
        return GetStatesResponse(
            description=f"Can't find such chat {request.chat_id}",
            status="error"
        )

    states = await ChatToolState.find_many(
        And(
            ChatToolState.chat_id == chat.id,
            ChatToolState.status == ChatToolStateStatus.in_progress
        )
    ).to_list()

    return GetStatesResponse(
        data=states,
        status="ok"
    )




import AgentService
from AgentService.utils.checkers import is_project, is_project_name
from AgentService.utils.templates import process_template

from shutil import ignore_patterns, copytree
import os
import click

import string


PROJECT_TEMPLATES = (
    ("agent.cfg", ),
)


IGNORE = ignore_patterns('*.pyc', '__pycache__', '.svn')


@click.group()
def group():
    pass


@group.command('create_nocode', help="Command that creates nocode AgentService project")
@click.argument('project_name', type=str)
def create_project_nocode(project_name):
    current_path = os.getcwd()
    templates_dir = os.path.join(AgentService.__path__[0], "templates", "project_nocode")
    project_dir = os.path.join(current_path, project_name)

    if is_project(current_path):
        return print(
            {"status": "err", "description": f"Error: AgentService project has already created in {current_path}"}
        )

    if os.path.isdir(project_name):
        return print(
            {"status": "err", "description": f"Error: directory with name {project_name} already exists in {project_dir}"}
        )

    if not is_project_name(project_name):
        return print(
            {"status": "err", "description": "Error: AgentService project names must begin with a letter and contain only letters, numbers and underscores"}
        )

    copytree(templates_dir, project_dir, ignore=IGNORE)

    for paths in PROJECT_TEMPLATES:
        file_path = os.path.join(
            project_dir,
            string.Template(os.path.join(*paths)).substitute(project_name=project_name)
        )
        process_template(file_path, project_name=project_name)

    return print(
        {"status": "ok", "description": f"AgentService project with name {project_name} has been created in {project_dir}"}
    )




import click
import subprocess
import os
from importlib.metadata import version

import AgentService


@click.group()
def group():
    pass


@group.command('image', help="Command that creates docker image")
def update_image():
    image_dir = os.path.join(AgentService.__path__[0], "templates", "agent-image")
    dockerfile_tmpl = os.path.join(image_dir, "Dockerfile.tmpl")
    dockerfile = os.path.join(image_dir, "Dockerfile")

    with open(dockerfile_tmpl, "r") as f:
        text = f.read()

    with open(dockerfile, "w") as f:
        f.write(text.format(version=version("AgentService")))

    subprocess.run(
        "docker build -t agentservice-base .",
        shell=True,
        text=True,
        cwd=image_dir
    )




import os
import re


def is_project_name(project_name: str) -> bool:
    return not re.search(r'^[_a-zA-Z]\w*$', project_name) is None


def is_project(project_path: os.PathLike) -> bool:
    return os.path.isfile(os.path.join(project_path, "agent.cfg"))
